MODULE agrif_lim2_interp
   !!======================================================================
   !!                       ***  MODULE agrif_lim2_update ***
   !! Nesting module :  update surface ocean boundary condition over ice
   !!                   from a child grif
   !! Sea-Ice model  :  LIM 2.0 Sea ice model time-stepping
   !!======================================================================
   !! History :  2.0   !  04-2008  (F. Dupont)  initial version
   !!            3.3   !  09-2010  (R. Benshila, C. Herbaut) update and EVP
   !!----------------------------------------------------------------------
#if defined key_agrif && defined key_lim2
   !!----------------------------------------------------------------------
   !!   'key_lim2'  :                                 LIM 2.0 sea-ice model
   !!   'key_agrif' :                                 AGRIF library
   !!----------------------------------------------------------------------
   !!   agrif_update_lim2  : update sea-ice model on boundaries or total
   !!                        sea-ice area
   !!----------------------------------------------------------------------
   USE ice_2
   USE dom_ice_2
   USE par_oce
   USE dom_oce
   USE agrif_ice

   IMPLICIT NONE
   PRIVATE

# if defined key_lim2_vp
   PUBLIC Agrif_dyn_lim2_load, Agrif_dyn_lim2_copy
# else
   PUBLIC Agrif_dyn_lim
# endif
   PUBLIC Agrif_adv_lim2
   PUBLIC Agrif_sadv_lim2
   PUBLIC interp_adv_ice, interp_sadv_ice
   PUBLIC interp_u_ice, interp_v_ice

   !!---------------------------------------------------------------------
   !! NEMO/NST 3.2 , LOCEAN-IPSL (2010)
   !! $Id$
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

# if defined key_lim2_vp
   SUBROUTINE agrif_dyn_lim2_load
      !!-----------------------------------------------------------------------
      !!              *** ROUTINE agrif_dyn_lim2_load ***
      !!
      !! need a special routine for dealing with exchanging data
      !! between the child and parent grid during ice step
      !!
      !!-----------------------------------------------------------------------
      !
      IF (Agrif_Root()) RETURN

      Agrif_SpecialValue=-9999.
      Agrif_UseSpecialValue = .TRUE.
      u_ice_nst(:,:) = 0.
      v_ice_nst(:,:) = 0.
      CALL Agrif_Bc_variable( u_ice_nst, u_ice_id ,procname=interp_u_ice, calledweight=1. )
      CALL Agrif_Bc_variable( v_ice_nst, v_ice_id ,procname=interp_v_ice, calledweight=1. )
      Agrif_SpecialValue=0.
      Agrif_UseSpecialValue = .FALSE.
      !
   END SUBROUTINE agrif_dyn_lim2_load


   SUBROUTINE agrif_dyn_lim2_copy(pu_n,pv_n)
      !!-----------------------------------------------------------------------
      !!                 *** ROUTINE agrif_dyn_lim2_copy ***
      !!-----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,0:jpj+1), INTENT(inout) :: pu_n, pv_n
      !!
      REAL(wp) :: zrhox, zrhoy
      INTEGER :: ji,jj
      !!-----------------------------------------------------------------------
      !
      IF (Agrif_Root()) RETURN

      zrhox = Agrif_Rhox()
      zrhoy = Agrif_Rhoy()

      IF((nbondi == -1).OR.(nbondi == 2)) THEN
            DO jj=2,jpj
               pu_n(3,jj) = u_ice_nst(3,jj)/(zrhoy*e2f(2,jj-1))*tmu(3,jj)
            END DO
            DO jj=2,jpj
               pv_n(3,jj) = v_ice_nst(3,jj)/(zrhox*e1f(2,jj-1))*tmu(3,jj)
            END DO
      ENDIF

      IF((nbondi == 1).OR.(nbondi == 2)) THEN
            DO jj=2,jpj
               pu_n(nlci-1,jj) = u_ice_nst(nlci-1,jj)/(zrhoy*e2f(nlci-2,jj-1))*tmu(nlci-1,jj)
            END DO
            DO jj=2,jpj
               pv_n(nlci-1,jj) = v_ice_nst(nlci-1,jj)/(zrhox*e1f(nlci-2,jj-1))*tmu(nlci-1,jj)
            END DO
      ENDIF

      IF((nbondj == -1).OR.(nbondj == 2)) THEN
            DO ji=2,jpi
               pv_n(ji,3) = v_ice_nst(ji,3)/(zrhox*e1f(ji-1,2))*tmu(ji,3)
            END DO
            DO ji=2,jpi
               pu_n(ji,3) = u_ice_nst(ji,3)/(zrhoy*e2f(ji-1,2))*tmu(ji,3)
            END DO
      ENDIF

      IF((nbondj == 1).OR.(nbondj == 2)) THEN
            DO ji=2,jpi
               pv_n(ji,nlcj-1) = v_ice_nst(ji,nlcj-1)/(zrhox*e1f(ji-1,nlcj-2))*tmu(ji,nlcj-1)
            END DO
            DO ji=2,jpi
               pu_n(ji,nlcj-1) = u_ice_nst(ji,nlcj-1)/(zrhoy*e2f(ji-1,nlcj-2))*tmu(ji,nlcj-1)
            END DO
      ENDIF
      !
   END SUBROUTINE agrif_dyn_lim2_copy

#else

   SUBROUTINE agrif_dyn_lim( kiter, kitermax, cd_type )
      !!-----------------------------------------------------------------------
      !!                    *** ROUTINE dyn_lim ***
      !!-----------------------------------------------------------------------
      INTEGER, INTENT(in) :: kiter, kitermax
      CHARACTER(len=1), INTENT( in ) :: cd_type
      !!
      INTEGER :: ji,jj
      REAL(wp) :: alpha, beta, zrhox, zrhoy
      REAL(wp), DIMENSION(:,:), ALLOCATABLE :: zuice, zvice
      !!-----------------------------------------------------------------------
      !
      IF (Agrif_Root()) RETURN
   
      IF( childfreq == 1. .AND. ( kiter == 0 .OR. kiter == 1 ) .AND. cd_type == 'V') THEN
        !
        ALLOCATE( zuice(jpi,jpj) , zvice(jpi,jpj) )
        ! switch old values
         IF(kiter > 0 ) THEN
            u_ice_oe(:,:,1) =  u_ice_oe(:,:,2)
            v_ice_oe(:,:,1) =  v_ice_oe(:,:,2)
            u_ice_sn(:,:,1) =  u_ice_sn(:,:,2)
            v_ice_sn(:,:,1) =  v_ice_sn(:,:,2)
         ENDIF
         ! interpolation of boundaries
         Agrif_SpecialValue=-9999.
         Agrif_UseSpecialValue = .TRUE.
         zuice = 0.
         zvice = 0.
         CALL Agrif_Bc_variable(zuice,u_ice_id,procname=interp_u_ice, calledweight=1.)
         CALL Agrif_Bc_variable(zvice,v_ice_id,procname=interp_v_ice, calledweight=1.)
         Agrif_SpecialValue=0.
         Agrif_UseSpecialValue = .FALSE.
         !  
         zrhox = agrif_rhox() ;    zrhoy = agrif_rhoy()      
         zuice(:,:) =  zuice(:,:)/(zrhoy*e2u(:,:))*umask(:,:,1)
         zvice(:,:) =  zvice(:,:)/(zrhox*e1v(:,:))*vmask(:,:,1)
         ! fill  boundaries
         DO jj = 1, jpj
         DO ji = 1, 2
            u_ice_oe(ji,jj,2) = zuice(ji,jj) ;  u_ice_oe(ji+2,jj,2) =  zuice(nlci+ji-3,jj)
         END DO
         END DO
         DO jj = 1, jpj
            v_ice_oe(  2,jj,2) = zvice(  2,jj) ;  v_ice_oe(  4,jj,2) =  zvice(       nlci-1,jj)
         END DO
         DO ji = 1, jpi
            u_ice_sn( ji,2,2) = zuice(ji,  2) ;  u_ice_sn( ji,4,2) =  zuice(ji,       nlcj-1)
         END DO
         DO jj = 1, 2
         DO ji = 1, jpi
            v_ice_sn(ji,jj,2) = zvice(ji,jj) ;  v_ice_sn(ji,jj+2,2) =  zvice(ji,nlcj+jj-3)
         END DO
         END DO
         !
         IF(kiter == 0 ) THEN
            u_ice_oe(:,:,1) =  u_ice_oe(:,:,2)
            v_ice_oe(:,:,1) =  v_ice_oe(:,:,2)
            u_ice_sn(:,:,1) =  u_ice_sn(:,:,2)
            v_ice_sn(:,:,1) =  v_ice_sn(:,:,2)
         ENDIF
         !`
         DEALLOCATE( zuice, zvice )
         !
      ENDIF 

      alpha = (childfreq ) / Agrif_Rhot()
      beta  = REAL(kiter,wp) / kitermax
      beta = alpha * beta
!RB   beta = 1 to put constant values on boundaries during EVP time-splitting    
!      beta=1
      SELECT CASE(cd_type)
      CASE('U')
         CALL ParcoursU(beta)
      CASE('V')
         CALL ParcoursV(beta)
      END SELECT
      !
   END SUBROUTINE agrif_dyn_lim


   SUBROUTINE ParcoursU( pbeta )
      !!-----------------------------------------------------------------------
      !!                    *** ROUTINE parcoursU ***
      !!-----------------------------------------------------------------------
      REAL(wp), INTENT(in) :: pbeta
      !!
      INTEGER :: ji, jj
      !!-----------------------------------------------------------------------
      !
      IF((nbondi == -1).OR.(nbondi == 2)) THEN
         DO jj=1,jpj
         DO ji=1,2
            u_ice(ji,jj) = (1-pbeta) * u_ice_oe(ji,jj,1) + pbeta * u_ice_oe(ji,jj,2)
         END DO
         END DO
         DO jj=1,jpj
            u_ice(2,jj) = 0.25*(u_ice(1,jj)+2.*u_ice(2,jj)+u_ice(3,jj))
            u_ice(2,jj) = u_ice(2,jj) * umask(2,jj,1)
         END DO
      ENDIF

      IF((nbondi == 1).OR.(nbondi == 2)) THEN
         DO jj=1,jpj
         DO ji=1,2
            u_ice(nlci+ji-3,jj) = (1-pbeta) * u_ice_oe(ji+2,jj,1) + pbeta * u_ice_oe(ji+2,jj,2)
         END DO
         END DO
         DO jj=1,jpj
            u_ice(nlci-2,jj) = 0.25*(u_ice(nlci-3,jj)+2.*u_ice(nlci-2,jj)+u_ice(nlci-1,jj))
            u_ice(nlci-2,jj) = u_ice(nlci-2,jj) * umask(nlci-2,jj,1)
         END DO
      ENDIF

      IF((nbondj == -1).OR.(nbondj == 2)) THEN
         DO ji=1,jpi
            u_ice(ji,2) = (1-pbeta) * u_ice_sn(ji,2,1) + pbeta * u_ice_sn(ji,2,2)
            u_ice(ji,2) = u_ice(ji,2)*umask(ji,2,1)
         END DO
      ENDIF

      IF((nbondj == 1).OR.(nbondj == 2)) THEN
         DO ji=1,jpi
            u_ice(ji,nlcj-1) = (1-pbeta) * u_ice_sn(ji,4,1) + pbeta * u_ice_sn(ji,4,2)
            u_ice(ji,nlcj-1) = u_ice(ji,nlcj-1)*umask(ji,nlcj-1,1)
         END DO
      ENDIF
      !
   END SUBROUTINE ParcoursU


   SUBROUTINE ParcoursV( pbeta )
      !!-----------------------------------------------------------------------
      !!                    *** ROUTINE parcoursV ***
      !!-----------------------------------------------------------------------
      REAL(wp), INTENT(in) :: pbeta
      !!
      INTEGER :: ji, jj
      !!-----------------------------------------------------------------------
      !
      IF((nbondi == -1).OR.(nbondi == 2)) THEN
        DO jj=1,jpj
            v_ice(2,jj) = (1-pbeta) * v_ice_oe(2,jj,1) + pbeta * v_ice_oe(2,jj,2)
            v_ice(2,jj) = v_ice(2,jj) * vmask(2,jj,1)
         END DO
      ENDIF

      IF((nbondi == 1).OR.(nbondi == 2)) THEN
        DO jj=1,jpj
            v_ice(nlci-1,jj) = (1-pbeta) * v_ice_oe(4,jj,1) + pbeta * v_ice_oe(4,jj,2)
            v_ice(nlci-1,jj) = v_ice(nlci-1,jj)*vmask(nlci-1,jj,1)
         END DO
      ENDIF

      IF((nbondj == -1).OR.(nbondj == 2)) THEN
         DO jj=1,2
         DO ji=1,jpi
            v_ice(ji,jj) = (1-pbeta) * v_ice_sn(ji,jj,1) + pbeta * v_ice_sn(ji,jj,2)
         END DO
         END DO
         DO ji=1,jpi
            v_ice(ji,2)=0.25*(v_ice(ji,1)+2.*v_ice(ji,2)+v_ice(ji,3))
            v_ice(ji,2)=v_ice(ji,2)*vmask(ji,2,1)
         END DO
      ENDIF

      IF((nbondj == 1).OR.(nbondj == 2)) THEN
         DO jj=1,2
         DO ji=1,jpi
            v_ice(ji,nlcj+jj-3) = (1-pbeta) * v_ice_sn(ji,jj+2,1) + pbeta * v_ice_sn(ji,jj+2,2)
         END DO
         END DO
         DO ji=1,jpi
            v_ice(ji,nlcj-2)=0.25*(v_ice(ji,nlcj-3)+2.*v_ice(ji,nlcj-2)+v_ice(ji,nlcj-1))
            v_ice(ji,nlcj-2) = v_ice(ji,nlcj-2) * vmask(ji,nlcj-2,1)
         END DO
      ENDIF
      !
   END SUBROUTINE ParcoursV
# endif

   SUBROUTINE agrif_adv_lim2( kt )
      !!-----------------------------------------------------------------------
      !!                  *** ROUTINE agrif_adv_lim2 ***
      !!-----------------------------------------------------------------------
      INTEGER, INTENT(in) :: kt
      !!
      INTEGER :: jk
      REAL(wp), DIMENSION(:,:,:), ALLOCATABLE :: ztab 
      !!-----------------------------------------------------------------------      
      !
      IF (Agrif_Root()) RETURN

      ALLOCATE( ztab(jpi,jpj,7) )

      ztab(:,:,:) = 0.
      Agrif_SpecialValue=-9999.
      Agrif_UseSpecialValue = .TRUE.
      CALL Agrif_Bc_variable( ztab, adv_ice_id ,procname=interp_adv_ice, calledweight=1. )
      Agrif_SpecialValue=0.
      Agrif_UseSpecialValue = .FALSE.
      !

      CALL parcoursT( ztab(:,:, 1), frld  )
      CALL parcoursT( ztab(:,:, 2), hicif )
      CALL parcoursT( ztab(:,:, 3), hsnif )
      CALL parcoursT( ztab(:,:, 4), tbif(:,:,1) )
      CALL parcoursT( ztab(:,:, 5), tbif(:,:,2) )
      CALL parcoursT( ztab(:,:, 6), tbif(:,:,3) )
      CALL parcoursT( ztab(:,:, 7), qstoif )
      !
      DEALLOCATE( ztab )
      !
   END SUBROUTINE agrif_adv_lim2


   SUBROUTINE agrif_sadv_lim2( kt )
      !!-----------------------------------------------------------------------
      !!                  *** ROUTINE agrif_sadv_lim2 ***
      !!-----------------------------------------------------------------------
      INTEGER, INTENT(in) :: kt
      !!
      INTEGER :: jk
      REAL(wp), DIMENSION(:,:,:), ALLOCATABLE :: ztab
      !!-----------------------------------------------------------------------      
      !
      IF (Agrif_Root()) RETURN

      ALLOCATE( ztab(jpi,jpj,42) )

      ztab(:,:,:) = 0.
      Agrif_SpecialValue=-9999.
      Agrif_UseSpecialValue = .TRUE.
      CALL Agrif_Bc_variable( ztab, sadv_ice_id ,procname=interp_sadv_ice, calledweight=1. )
      Agrif_SpecialValue=0.
      Agrif_UseSpecialValue = .FALSE.
      !
      DO jk = 1, 42
         ztab(:,:,jk) = ztab(:,:,jk) * area(:,:)
      END DO

      CALL parcoursT( ztab(:,:, 1), sxice  )
      CALL parcoursT( ztab(:,:, 2), syice  )
      CALL parcoursT( ztab(:,:, 3), sxxice )
      CALL parcoursT( ztab(:,:, 4), syyice )
      CALL parcoursT( ztab(:,:, 5), sxyice )
      CALL parcoursT( ztab(:,:, 6), sxa    )
      CALL parcoursT( ztab(:,:, 7), sya    )
      CALL parcoursT( ztab(:,:, 8), sxxa   )
      CALL parcoursT( ztab(:,:, 9), syya   )
      CALL parcoursT( ztab(:,:,10), sxya   )
      CALL parcoursT( ztab(:,:,11), sxsn   )
      CALL parcoursT( ztab(:,:,12), sysn   )
      CALL parcoursT( ztab(:,:,13), sxxsn  )
      CALL parcoursT( ztab(:,:,14), syysn  )
      CALL parcoursT( ztab(:,:,15), sxysn  )
      CALL parcoursT( ztab(:,:,16), sxc0   )
      CALL parcoursT( ztab(:,:,17), syc0   )
      CALL parcoursT( ztab(:,:,18), sxxc0  )
      CALL parcoursT( ztab(:,:,19), syyc0  )
      CALL parcoursT( ztab(:,:,20), sxyc0  )
      CALL parcoursT( ztab(:,:,21), sxc1   )
      CALL parcoursT( ztab(:,:,22), syc1   )
      CALL parcoursT( ztab(:,:,23), sxxc1  )
      CALL parcoursT( ztab(:,:,24), syyc1  )
      CALL parcoursT( ztab(:,:,25), sxyc1  )
      CALL parcoursT( ztab(:,:,26), sxc2   )
      CALL parcoursT( ztab(:,:,27), syc2   )
      CALL parcoursT( ztab(:,:,28), sxxc2  )
      CALL parcoursT( ztab(:,:,29), syyc2  )
      CALL parcoursT( ztab(:,:,30), sxyc2  )
      CALL parcoursT( ztab(:,:,31), sxst   )
      CALL parcoursT( ztab(:,:,32), syst   )
      CALL parcoursT( ztab(:,:,33), sxxst  )
      CALL parcoursT( ztab(:,:,34), syyst  )
      CALL parcoursT( ztab(:,:,35), sxyst  )

      CALL parcoursT( ztab(:,:,36), s0ice  )
      CALL parcoursT( ztab(:,:,37), s0a  )
      CALL parcoursT( ztab(:,:,38), s0sn  )
      CALL parcoursT( ztab(:,:,39), s0c0  )
      CALL parcoursT( ztab(:,:,40), s0c1  )
      CALL parcoursT( ztab(:,:,41), s0c2  )
      CALL parcoursT( ztab(:,:,42), s0st  )
      !
      DEALLOCATE( ztab )
      !
   END SUBROUTINE agrif_sadv_lim2


   SUBROUTINE parcoursT ( pinterp, pfinal )
      !!-----------------------------------------------------------------------
      !!                    *** ROUTINE parcoursT ***
      !!-----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj), INTENT(in)    :: pinterp
      REAL(wp), DIMENSION(jpi,jpj), INTENT(inout) :: pfinal
      !!
      INTEGER :: ji, jj
      REAL(wp) :: zrho
      REAL(wp) :: zbound, zvbord
      REAL(wp) :: alpha1, alpha2, alpha3, alpha4
      REAL(wp) :: alpha5, alpha6, alpha7
      REAL(wp), DIMENSION(:,:), ALLOCATABLE :: zui_u, zvi_v
      !!-----------------------------------------------------------------------
      !
      ALLOCATE( zui_u(jpi,jpj) ,zvi_v(jpi,jpj) )

      zrho = Agrif_Rhox()
      alpha1 = ( zrho -1. ) / 2.
      alpha2 = 1. - alpha1
      alpha3 = ( zrho - 1 ) / ( zrho + 1 )
      alpha4 = 1. - alpha3
      alpha6 = 2. * ( zrho -1. ) / ( zrho + 1. )
      alpha7 = -( zrho -1 ) / ( zrho + 3 )
      alpha5 = 1. - alpha6 - alpha7

      zui_u = 0.e0
      zvi_v = 0.e0
      ! zvbord factor between 1 and 2 to take into account slip or no-slip boundary conditions.        
      zbound=0.
      zvbord = 1.0 + ( 1.0 - zbound )
#if defined key_lim2_vp
      DO jj = 1, jpjm1
         DO ji = 1, jpim1
            zui_u(ji,jj) = ( u_ice(ji+1,jj  ) + u_ice(ji+1,jj+1) ) / ( MAX( tmu(ji+1,jj  ) + tmu(ji+1,jj+1), zvbord ) )
            zvi_v(ji,jj) = ( v_ice(ji  ,jj+1) + v_ice(ji+1,jj+1) ) / ( MAX( tmu(ji  ,jj+1) + tmu(ji+1,jj+1), zvbord ) )
         END DO
      END DO
#else
      zui_u(:,:) = u_ice(:,:)
      zvi_v(:,:) = v_ice(:,:)
#endif

      IF((nbondi == 1).OR.(nbondi == 2)) THEN
         pfinal (nlci,:) = alpha1 * pinterp(nlci,:) + alpha2 * pinterp(nlci-1,:)
         DO jj=1,jpj
            IF (umask(nlci-2,jj,1).EQ.0.) THEN
               pfinal (nlci-1,jj) = pfinal (nlci,jj) * tms(nlci-1,jj)
            ELSE
               pfinal (nlci-1,jj)=(alpha4*pfinal (nlci,jj)+alpha3*pfinal (nlci-2,jj))*tms(nlci-1,jj)
               IF (zui_u(nlci-2,jj).GT.0.) THEN
         	  pfinal(nlci-1,jj)=( alpha6*pfinal (nlci-2,jj)+alpha5*pfinal (nlci,jj)  &
         			    + alpha7*pfinal (nlci-3,jj) ) * tms(nlci-1,jj)
               ENDIF
            ENDIF
         END DO
      ENDIF

      IF((nbondj == 1).OR.(nbondj == 2)) THEN
         pfinal (:,nlcj) = alpha1 * pinterp(:,nlcj) + alpha2 * pinterp(:,nlcj-1)
         DO ji=1,jpi
            IF (vmask(ji,nlcj-2,1).EQ.0.) THEN
               pfinal (ji,nlcj-1) =  pfinal(ji,nlcj) * tms(ji,nlcj-1)
            ELSE
               pfinal (ji,nlcj-1)=(alpha4*pfinal (ji,nlcj)+alpha3*pfinal (ji,nlcj-2))*tms(ji,nlcj-1)
               IF (zvi_v(ji,nlcj-2) .GT. 0.) THEN
         	  pfinal (ji,nlcj-1)=( alpha6*pfinal (ji,nlcj-2)+alpha5*pfinal (ji,nlcj)  &
         			    + alpha7*pfinal (ji,nlcj-3) ) * tms(ji,nlcj-1)
               ENDIF
            ENDIF
         END DO
      ENDIF

      IF((nbondi == -1).OR.(nbondi == 2)) THEN
         pfinal (1,:) = alpha1 * pinterp(1,:) + alpha2 * pinterp(2,:)
         DO jj=1,jpj
            IF (umask(2,jj,1).EQ.0.) THEN
               pfinal (2,jj) = pfinal (1,jj) * tms(2,jj)
            ELSE
               pfinal (2,jj)=(alpha4*pfinal (1,jj)+alpha3*pfinal (3,jj))*tms(2,jj)	   
              IF (zui_u(2,jj).LT.0.) THEN
         	  pfinal (2,jj)=(alpha6*pfinal (3,jj)+alpha5*pfinal (1,jj)+alpha7*pfinal (4,jj))*tms(2,jj)
               ENDIF
            ENDIF
         END DO
      ENDIF

      IF((nbondj == -1).OR.(nbondj == 2)) THEN
         pfinal (:,1) = alpha1 * pinterp(:,1) + alpha2 * pinterp(:,2)
         DO ji=1,jpi
            IF (vmask(ji,2,1).EQ.0.) THEN
               pfinal (ji,2) = pfinal (ji,1) * tms(ji,2)
            ELSE
               pfinal (ji,2)=(alpha4*pfinal (ji,1)+alpha3*pfinal (ji,3))*tms(ji,2)	   
               IF (zvi_v(ji,2) .LT. 0.) THEN
         	  pfinal (ji,2)=(alpha6*pfinal (ji,3)+alpha5*pfinal (ji,1)+alpha7*pfinal (ji,4))*tms(ji,2)
               ENDIF
            ENDIF
         END DO
      ENDIF

      pfinal (:,:) = pfinal (:,:) * tms(:,:)
      ! 
   END SUBROUTINE parcoursT


   SUBROUTINE interp_u_ice( tabres, i1, i2, j1, j2 )
      !!-----------------------------------------------------------------------
      !!                     *** ROUTINE interp_u_ice ***
      !!-----------------------------------------------------------------------
      INTEGER, INTENT(in) :: i1, i2, j1, j2
      REAL(wp), DIMENSION(i1:i2,j1:j2), INTENT(inout) :: tabres
      !!
      INTEGER :: ji,jj
      !!-----------------------------------------------------------------------
      !
#if defined key_lim2_vp
      DO jj=MAX(j1,2),j2
         DO ji=MAX(i1,2),i2
            IF( tmu(ji,jj) == 0. ) THEN
               tabres(ji,jj) = -9999.
            ELSE
               tabres(ji,jj) = e2f(ji-1,jj-1) * u_ice(ji,jj)
            ENDIF
         END DO
      END DO
#else
      DO jj= j1, j2
         DO ji= i1, i2
            IF( umask(ji,jj,1) == 0. ) THEN
               tabres(ji,jj) = -9999.
            ELSE
               tabres(ji,jj) = e2u(ji,jj) * u_ice(ji,jj)
            ENDIF
         END DO
      END DO
#endif
   END SUBROUTINE interp_u_ice


   SUBROUTINE interp_v_ice( tabres, i1, i2, j1, j2 )
      !!-----------------------------------------------------------------------
      !!                    *** ROUTINE interp_v_ice ***
      !!-----------------------------------------------------------------------      
      INTEGER, INTENT(in) :: i1, i2, j1, j2
      REAL(wp), DIMENSION(i1:i2,j1:j2), INTENT(inout) :: tabres
      !!
      INTEGER :: ji, jj
      !!-----------------------------------------------------------------------
      !
#if defined key_lim2_vp
      DO jj=MAX(j1,2),j2
         DO ji=MAX(i1,2),i2
            IF( tmu(ji,jj) == 0. ) THEN
               tabres(ji,jj) = -9999.
            ELSE
               tabres(ji,jj) = e1f(ji-1,jj-1) * v_ice(ji,jj)
            ENDIF
         END DO
      END DO
#else
      DO jj= j1 ,j2
         DO ji = i1, i2
            IF( vmask(ji,jj,1) == 0. ) THEN
               tabres(ji,jj) = -9999.
            ELSE
               tabres(ji,jj) = e1v(ji,jj) * v_ice(ji,jj)
            ENDIF
         END DO
      END DO
#endif
   END SUBROUTINE interp_v_ice


   SUBROUTINE interp_adv_ice( tabres, i1, i2, j1, j2 )
      !!-----------------------------------------------------------------------
      !!                    *** ROUTINE interp_adv_ice ***                           
      !!
      !! ** Purpose : fill an array with  ice variables
      !!              to be advected
      !!              put -9999 where no ice for correct extrapolation             
      !!-----------------------------------------------------------------------
      INTEGER, INTENT(in) :: i1, i2, j1, j2
      REAL(wp), DIMENSION(i1:i2,j1:j2,7), INTENT(inout) :: tabres
      !!
      INTEGER :: ji, jj, jk
      !!-----------------------------------------------------------------------
      !
      DO jj=j1,j2
         DO ji=i1,i2
            IF( tms(ji,jj) == 0. ) THEN
               tabres(ji,jj,:) = -9999. 
            ELSE
               tabres(ji,jj, 1) = frld  (ji,jj)
               tabres(ji,jj, 2) = hicif (ji,jj)
               tabres(ji,jj, 3) = hsnif (ji,jj)
               tabres(ji,jj, 4) = tbif  (ji,jj,1)
               tabres(ji,jj, 5) = tbif  (ji,jj,2)
               tabres(ji,jj, 6) = tbif  (ji,jj,3)
               tabres(ji,jj, 7) = qstoif(ji,jj)
            ENDIF
         END DO
      END DO
      !
   END SUBROUTINE interp_adv_ice


   SUBROUTINE interp_sadv_ice( tabres, i1, i2, j1, j2 )
      !!-----------------------------------------------------------------------
      !!                 *** ROUTINE interp_sadv_ice ***   
      !!
      !! ** Purpose : fill an array with superior moements of ice variables
      !!              to be advected
      !!              put -9990 where no ice for correct extrapolation             
      !!-----------------------------------------------------------------------
      INTEGER, INTENT(in) :: i1, i2, j1, j2
      REAL(wp), DIMENSION(i1:i2,j1:j2,42), INTENT(inout) :: tabres
      !!
      INTEGER :: ji, jj, jk
      REAL(wp) :: z1_area
      !!-----------------------------------------------------------------------
      !
      DO jj=j1,j2
         DO ji=i1,i2
            IF( tms(ji,jj) == 0. ) THEN
               tabres(ji,jj,:) = -9999. 
            ELSE
               z1_area = 1. / area(ji,jj)
               tabres(ji,jj, 1) = sxice (ji,jj) * z1_area
               tabres(ji,jj, 2) = syice (ji,jj) * z1_area  
               tabres(ji,jj, 3) = sxxice(ji,jj) * z1_area
               tabres(ji,jj, 4) = syyice(ji,jj) * z1_area
               tabres(ji,jj, 5) = sxyice(ji,jj) * z1_area
               tabres(ji,jj, 6) = sxa   (ji,jj) * z1_area
               tabres(ji,jj, 7) = sya   (ji,jj) * z1_area
               tabres(ji,jj, 8) = sxxa  (ji,jj) * z1_area
               tabres(ji,jj, 9) = syya  (ji,jj) * z1_area
               tabres(ji,jj,10) = sxya  (ji,jj) * z1_area
               tabres(ji,jj,11) = sxsn  (ji,jj) * z1_area
               tabres(ji,jj,12) = sysn  (ji,jj) * z1_area
               tabres(ji,jj,13) = sxxsn (ji,jj) * z1_area
               tabres(ji,jj,14) = syysn (ji,jj) * z1_area
               tabres(ji,jj,15) = sxysn (ji,jj) * z1_area
               tabres(ji,jj,16) = sxc0  (ji,jj) * z1_area
               tabres(ji,jj,17) = syc0  (ji,jj) * z1_area
               tabres(ji,jj,18) = sxxc0 (ji,jj) * z1_area
               tabres(ji,jj,19) = syyc0 (ji,jj) * z1_area
               tabres(ji,jj,20) = sxyc0 (ji,jj) * z1_area
               tabres(ji,jj,21) = sxc1  (ji,jj) * z1_area
               tabres(ji,jj,22) = syc1  (ji,jj) * z1_area
               tabres(ji,jj,23) = sxxc1 (ji,jj) * z1_area
               tabres(ji,jj,24) = syyc1 (ji,jj) * z1_area
               tabres(ji,jj,25) = sxyc1 (ji,jj) * z1_area
               tabres(ji,jj,26) = sxc2  (ji,jj) * z1_area
               tabres(ji,jj,27) = syc2  (ji,jj) * z1_area
               tabres(ji,jj,28) = sxxc2 (ji,jj) * z1_area
               tabres(ji,jj,29) = syyc2 (ji,jj) * z1_area
               tabres(ji,jj,30) = sxyc2 (ji,jj) * z1_area
               tabres(ji,jj,31) = sxst  (ji,jj) * z1_area
               tabres(ji,jj,32) = syst  (ji,jj) * z1_area
               tabres(ji,jj,33) = sxxst (ji,jj) * z1_area
               tabres(ji,jj,34) = syyst (ji,jj) * z1_area
               tabres(ji,jj,35) = sxyst (ji,jj) * z1_area

               tabres(ji,jj,36) = s0ice (ji,jj)! * z1_area
               tabres(ji,jj,37) = s0a   (ji,jj)! * z1_area
               tabres(ji,jj,38) = s0sn  (ji,jj)! * z1_area
               tabres(ji,jj,39) = s0c0  (ji,jj)! * z1_area
               tabres(ji,jj,40) = s0c1  (ji,jj)! * z1_area
               tabres(ji,jj,41) = s0c2  (ji,jj)! * z1_area
               tabres(ji,jj,42) = s0st  (ji,jj)! * z1_area
            ENDIF
         END DO
      END DO
      !
   END SUBROUTINE interp_sadv_ice

#else
CONTAINS
   SUBROUTINE agrif_lim2_interp_empty
      !!---------------------------------------------
      !!   *** ROUTINE agrif_lim2_interp_empty ***
      !!---------------------------------------------
      WRITE(*,*)  'agrif_lim2_interp : You should not have seen this print! error?'
   END SUBROUTINE agrif_lim2_interp_empty
#endif
END MODULE agrif_lim2_interp
